#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella loader implementation for Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"


import os
import inspect
import logging.config


def init(do_reload=False, import_libs=True, dev=False, artella_configs_path=None):
    """
    Initializes Plot Twist library
    """

    # Load logger configuration
    logging.config.fileConfig(get_logging_config(), disable_existing_loggers=False)

    # Without default_integrations=False, PyInstaller fails during launcher generation
    if not dev:
        import sentry_sdk
        try:
            sentry_sdk.init("https://d71a4ba272374d7fb845269bb2aebf37@sentry.io/1816355")
        except (RuntimeError, ImportError):
            sentry_sdk.init("https://d71a4ba272374d7fb845269bb2aebf37@sentry.io/1816355", default_integrations=False)

    from tpPyUtils import importer

    class PlotTwistCore(importer.Importer, object):
        def __init__(self):
            super(PlotTwistCore, self).__init__(module_name='plottwist')

        def get_module_path(self):
            """
            Returns path where plot twist core module is stored
            :return: str
            """

            try:
                mod_dir = os.path.dirname(inspect.getframeinfo(inspect.currentframe()).filename)
            except Exception:
                try:
                    mod_dir = os.path.dirname(__file__)
                except Exception:
                    try:
                        import tpDccLib
                        mod_dir = tpDccLib.__path__[0]
                    except Exception:
                        return None

            return mod_dir

    packages_order = [
        'plottwist.core'
    ]

    if import_libs:
        import tpPyUtils
        tpPyUtils.init(do_reload=do_reload)
        import tpDccLib
        tpDccLib.init(do_reload=do_reload)
        import tpQtLib
        tpQtLib.init(do_reload=do_reload)
        import tpNameIt
        tpNameIt.init(do_reload=do_reload)
        import artellapipe.loader
        artellapipe.loader.init(do_reload=do_reload, dev=dev)

    from plottwist.core import project

    plottwist_core_importer = importer.init_importer(importer_class=PlotTwistCore, do_reload=False)
    plottwist_core_importer.import_packages(
        only_packages=False,
        order=packages_order,
        skip_modules=['plottwist.bootstrap']
    )
    if do_reload:
        plottwist_core_importer.reload_all()

    create_logger_directory()

    from artellapipe.utils import resource
    resources_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'resources')
    resource.ResourceManager().register_resource(resources_path, 'project')
    resource.ResourceManager().register_resource(os.path.join(resources_path, 'icons', 'shelf'), 'shelf')

    import artellapipe.loader
    artellapipe.loader.set_project(project.PlotTwist, do_reload=do_reload)


def create_logger_directory():
    """
    Creates artellapipe logger directory
    """

    artellapipe_logger_dir = os.path.normpath(os.path.join(os.path.expanduser('~'), 'plottwist', 'logs'))
    if not os.path.isdir(artellapipe_logger_dir):
        os.makedirs(artellapipe_logger_dir)


def get_logging_config():
    """
    Returns logging configuration file path
    :return: str
    """

    create_logger_directory()

    return os.path.normpath(os.path.join(os.path.dirname(__file__), '__logging__.ini'))


def get_logging_level():
    """
    Returns logging level to use
    :return: str
    """

    if os.environ.get('PLOTTWIST_LOG_LEVEL', None):
        return os.environ.get('PLOTTWIST_LOG_LEVEL')

    return os.environ.get('PLOTTWIST_LOG_LEVEL', 'WARNING')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains core classes for tools
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import logging

import artellapipe.register
from artellapipe.core import tool

LOGGER = logging.getLogger()


class PlotTwistTool(tool.Tool, object):
    def __init__(self, project, config, parent=None):
        super(PlotTwistTool, self).__init__(project=project, config=config, parent=parent)

    def set_attacher(self, attacher):
        super(PlotTwistTool, self).set_attacher(attacher=attacher)

        if self._attacher and hasattr(self._attacher, 'try_kitsu_login'):
            LOGGER.info('Trying to login into Kitsu ...')
            self._attacher.try_kitsu_login()


artellapipe.register.register_class('Tool', PlotTwistTool)



# -*- coding: utf-8 -*-

"""
Module that contains widgets related with Plot Twist assets
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import logging

import artellapipe.register
from artellapipe.widgets import asset

LOGGER = logging.getLogger()


class PlotTwistAssetWidget(asset.ArtellaAssetWidget, object):
    def __init__(self, asset, text=None, parent=None):
        super(PlotTwistAssetWidget, self).__init__(asset=asset, text=text, parent=parent)


artellapipe.register.register_class('AssetWidget', PlotTwistAssetWidget)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains definitions for character assets in Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from plottwist.core import asset


class PlotTwistCharacter(asset.PlotTwistAsset, object):
    def __init__(self, project, asset_data):
        super(PlotTwistCharacter, self).__init__(project=project, asset_data=asset_data)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for Solstice Artella project
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"


from artellapipe.core import project as artella_project


class PlotTwist(artella_project.ArtellaProject, object):
    def __init__(self):
        super(PlotTwist, self).__init__(name='PlotTwist')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains manager that handles Plot Twist DCC Menu
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import webbrowser

from Qt.QtWidgets import *

from tpPyUtils import decorators

import artellapipe.register
from artellapipe.utils import resource
import artellapipe.libs.kitsu as kitsu_lib


class PlotTwistMenu(artellapipe.Menu, object):
    def __init__(self):

        self._kitsu_action_object_name = None

        super(PlotTwistMenu, self).__init__()

    def set_project(self, project):
        self._kitsu_action_object_name = '{}_kitsu'.format(self._menu_name)
        super(PlotTwistMenu, self).set_project(project)

    def create_menus(self):
        valid_creation = super(PlotTwistMenu, self).create_menus()
        if not valid_creation:
            return False

        self.create_kitsu_menu()

    def get_menu_names(self):
        menu_names = super(PlotTwistMenu, self).get_menu_names()
        if self._kitsu_action_object_name not in menu_names:
            menu_names.append(self._kitsu_action_object_name)

        return menu_names

    def create_kitsu_menu(self):
        self._kitsu_action = QAction(self._parent.menuBar())
        self._kitsu_action.setIcon(resource.ResourceManager().icon('kitsu'))
        self._parent.menuBar().addAction(self._kitsu_action)
        self._kitsu_action.setObjectName(self._kitsu_action_object_name)
        self._kitsu_action.triggered.connect(self._on_kitsu_open)

    def _on_kitsu_open(self):
        """
        Internal callback function that is called when kitsu action is pressed
        """

        project_url = kitsu_lib.config.get('project_url', None)
        if not project_url:
            return None

        webbrowser.open(project_url)


@decorators.Singleton
class PlotTwistMenuManagerSingleton(PlotTwistMenu, object):
    def __init__(self):
        PlotTwistMenu.__init__(self)


artellapipe.register.register_class('MenuMgr', PlotTwistMenuManagerSingleton)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Base wrapper classes to create DCC windows
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import artellapipe.register
from artellapipe.widgets import window
from artellapipe.libs.kitsu.widgets import userinfo


class PlotTwistWindowStatusBar(window.ArtellaWindowStatusBar, object):
    def __init__(self, parent=None):
        super(PlotTwistWindowStatusBar, self).__init__(parent=parent)

        self._user_info = userinfo.KitsuUserInfo(project=self._project, parent=parent)
        self.main_layout.insertWidget(self.main_layout.count() - 1, self._user_info)

    def set_project(self, project):
        super(PlotTwistWindowStatusBar, self).set_project(project)

        self._user_info.set_project(project)

    def try_kitsu_login(self):
        """
        Function that tries to login into Kitsu with stored credentials
        :return: bool
        """

        valid_login = self._user_info.try_kitsu_login()
        if valid_login:
            return True

        return False


class PlotTwistWindow(window.ArtellaWindow, object):

    STATUS_BAR_WIDGET = PlotTwistWindowStatusBar

    def __init__(self, *args, **kwargs):
        super(PlotTwistWindow, self).__init__(*args, **kwargs)

    def try_kitsu_login(self):
        """
        Function that tries to login into Kitsu with stored credentials
        :return: bool
        """

        return self._status_bar.try_kitsu_login()


artellapipe.register.register_class('Window', PlotTwistWindow)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementations for rig asset files
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from artellapipe.core import assetfile


class PlotTwistRigAssetFile(assetfile.ArtellaAssetFile, object):
    def __init__(self, asset):
        super(PlotTwistRigAssetFile, self).__init__(file_asset=asset)



# -*- coding: utf-8 -*-

"""
Module that contains definitions for asset in Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import logging

import artellapipe.register
from artellapipe.core import asset

LOGGER = logging.getLogger()


class PlotTwistAsset(asset.ArtellaAsset, object):
    def __init__(self, project, asset_data, node=None):
        super(PlotTwistAsset, self).__init__(project=project, asset_data=asset_data, node=node)

    def reference_rig_file(self, file_type, sync=False):
        """
        References rig file of the current asset
        :param file_type:
        :param sync:
        :return:
        """

        return self.reference_file(
            file_type=file_type, namespace=self.get_id(),
            status=asset.ArtellaAssetFileStatus.PUBLISHED, sync=sync)


artellapipe.register.register_class('Asset', PlotTwistAsset)
